@@ -1,7 +1,7 @@
 /*
  * MP3/MPlayer plugin to VDR (C++)
  *
- * (C) 2001-2004 Stefan Huelswitt <s.huelswitt@gmx.de>
+ * (C) 2001-2010 Stefan Huelswitt <s.huelswitt@gmx.de>
  *
  * This code is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,9 @@
 #include <getopt.h>
 #include <malloc.h>
 #include <stdlib.h>
+#include <ctype.h>
+
+#include "common.h"
 
 #include <vdr/plugin.h>
 #include <vdr/player.h>
@@ -29,32 +32,32 @@
 #include <vdr/font.h>
 #include <vdr/osdbase.h>
 #include <vdr/menuitems.h>
-#ifdef HAVE_BEAUTYPATCH
-#include <vdr/fontsym.h>
-#endif
-#ifndef VDRVERSNUM
-#include <vdr/config.h>
-#endif
-#if VDRVERSNUM >= 10307
 #include <vdr/skins.h>
-#endif
+#include <vdr/remote.h>
+#include <vdr/menu.h>
 
-#include "common.h"
 #include "setup.h"
 #include "setup-mplayer.h"
 #include "menu.h"
 #include "player-mplayer.h"
 #include "data.h"
+#include "data-src.h"
 #include "i18n.h"
 #include "version.h"
+#include "service.h"
 
+const char *sourcesSub=0;
 cFileSources MPlaySources;
 
+static const char *plugin_name=0;
+const char *i18n_name=0;
+
 // --- cMenuSetupMPlayer --------------------------------------------------------
 
 class cMenuSetupMPlayer : public cMenuSetupPage {
 private:
   cMPlayerSetup data;
+  const char *res[3];
 protected:
   virtual void Store(void);
 public:
@@ -66,8 +69,18 @@ cMenuSetupMPlayer::cMenuSetupMPlayer(void)
   data=MPlayerSetup;
   SetSection(tr("MPlayer"));
   Add(new cMenuEditBoolItem(tr("Setup.MPlayer$Control mode"),  &data.SlaveMode, tr("Traditional"), tr("Slave")));
-  Add(new cMenuEditBoolItem(tr("Setup.MPlayer$Disable resume"),&data.DisableResume));
+  Add(new cMenuEditBoolItem(tr("Setup.MPlayer$Prev/Next keys"),&data.PrevNextKeyMode, tr("Chapter"), tr("Playlist")));
+  res[0]=tr("disabled");
+  res[1]=tr("global only");
+  res[2]=tr("local first");
+  Add(new cMenuEditStraItem(tr("Setup.MPlayer$Resume mode"),   &data.ResumeMode, 3, res));
   Add(new cMenuEditBoolItem(tr("Hide mainmenu entry"),         &data.HideMainMenu));
+  for(int i=0; i<10; i++) {
+    char name[32];
+    snprintf(name,sizeof(name),"%s %d",tr("Setup.MPlayer$Slave command key"),i);
+    static const char allowed[] = { "abcdefghijklmnopqrstuvwxyz0123456789!\"$%&/()=?{}[]\\+*~#',;.:-_<>|@`^" };
+    Add(new cMenuEditStrItem(name, data.KeyCmd[i],MAX_KEYCMD,allowed));
+    }
 }
 
 void cMenuSetupMPlayer::Store(void)
@@ -75,6 +88,13 @@ void cMenuSetupMPlayer::Store(void)
   MPlayerSetup=data;
   SetupStore("ControlMode", MPlayerSetup.SlaveMode);
   SetupStore("HideMainMenu",MPlayerSetup.HideMainMenu);
+  SetupStore("ResumeMode",  MPlayerSetup.ResumeMode);
+  SetupStore("PrevNextMode",MPlayerSetup.PrevNextKeyMode);
+  for(int i=0; i<10; i++) {
+    char name[16];
+    snprintf(name,sizeof(name),"KeyCmd%d",i);
+    SetupStore(name,MPlayerSetup.KeyCmd[i]);
+    }
 }
 
 // --- cMPlayerControl ---------------------------------------------------------
@@ -84,21 +104,20 @@ private:
   static cFileObj *file;
   static bool rewind;
   cMPlayerPlayer *player;
-#if VDRVERSNUM >= 10307
   cSkinDisplayReplay *display;
-#endif
-  bool visible, modeOnly, haveBeauty;
+  bool visible, modeOnly;
   time_t timeoutShow;
-  int lastCurrent, lastTotal, osdPos;
+  int lastCurrent, lastTotal;
+  char *lastReplayMsg;
   //
   bool jumpactive, jumphide, jumpmode;
   int jumpval;
   //
   void Stop(void);
   void ShowTimed(int Seconds=0);
-  void DisplayAtBottom(const char *s);
   void ShowProgress(void);
   void ShowMode(void);
+  void ShowTitle(void);
   void Jump(void);
   void JumpProcess(eKeys Key);
   void JumpDisplay(void);
@@ -117,35 +136,17 @@ bool cMPlayerControl::rewind=false;
 cMPlayerControl::cMPlayerControl(void)
 :cControl(player=new cMPlayerPlayer(file,rewind))
 {
-  visible=modeOnly=jumpactive=haveBeauty=false; osdPos=0;
-  cStatus::MsgReplaying(this,file->Path());
-#if VDRVERSNUM >= 10307
+  visible=modeOnly=jumpactive=false;
+  lastReplayMsg=0;
   display=0;
-#else
-#ifdef HAVE_BEAUTYPATCH
-#if VDRVERSNUM >= 10300
-  const cFont *sym=cFont::GetFont(fontSym);
-  const cFont *osd=cFont::GetFont(fontOsd);
-  const cFont::tCharData *symD=sym->CharData(32);
-  const cFont::tCharData *osdD=osd->CharData(32);
-#else //VDRVERSNUM >= 10300
-  cFont *sym=new cFont(fontSym);
-  cFont *osd=new cFont(fontOsd);
-  const cFont::tCharData *symD=sym->CharData(32);
-  const cFont::tCharData *osdD=osd->CharData(32);
-  delete sym;
-  delete osd;
-#endif //VDRVERSNUM >= 10300
-  if(symD != osdD) haveBeauty=true;
-  d(printf("mplayer: beauty patch %sdetected\n",haveBeauty?"":"NOT "))
-#endif //HAVE_BEAUTYPATCH
-#endif //VDRVERSNUM >= 10307
+  ShowTitle();
 }
 
 cMPlayerControl::~cMPlayerControl()
 {
   Stop();
-  cStatus::MsgReplaying(this, NULL);
+  cStatus::MsgReplaying(this,0,0,false);
+  free(lastReplayMsg);
 }
 
 void cMPlayerControl::SetFile(const cFileObj *File, bool Rewind)
@@ -172,27 +173,38 @@ void cMPlayerControl::ShowTimed(int Seconds)
 void cMPlayerControl::Hide(void)
 {
   if(visible) {
-#if VDRVERSNUM >= 10307
     delete display; display=0;
+    visible=modeOnly=false;
+#if APIVERSNUM >= 10500
+    SetNeedsFastResponse(false);
 #else
-    Interface->Close();
+    needsFastResponse=false;
 #endif
-    needsFastResponse=visible=modeOnly=false;
     }
 }
 
-void cMPlayerControl::DisplayAtBottom(const char *s)
+void cMPlayerControl::ShowTitle(void)
 {
-#if VDRVERSNUM < 10307
-  const int p=modeOnly ? 0 : 2;
-  if(s) {
-    const int d=max(Width()-cOsd::WidthInCells(s),0) / 2;
-    if(modeOnly) Interface->Fill(0, p, Interface->Width(), 1, clrTransparent);
-    Interface->Write(d, p, s);
+  const char *path=0;
+  bool release=true;
+  if(player) path=player->GetCurrentName();
+  if(!path) {
+    path=file->FullPath();
+    release=false;
     }
-  else
-    Interface->Fill(12, p, Width() - 22, 1, clrBackground);
-#endif
+  if(path) {
+    const char *name=rindex(path,'/');
+    if(name) name++; else name=path;
+    if(!lastReplayMsg || strcmp(lastReplayMsg,path)) {
+      cStatus::MsgReplaying(this,name,path,true);
+      free(lastReplayMsg);
+      lastReplayMsg=strdup(path);
+      }
+    if(visible) {
+      if(display) display->SetTitle(name);
+      }
+    }
+  if(release) free((void *)path);
 }
 
 void cMPlayerControl::ShowProgress(void)
@@ -202,68 +214,35 @@ void cMPlayerControl::ShowProgress(void)
   if(GetIndex(Current,Total) && Total>0) {
     bool flush=false;
     if(!visible) {
-#if VDRVERSNUM >= 10307
-      display=Skins.Current()->DisplayReplay(modeOnly);
+      display=Skins.Current()->DisplayReplay(false);
+      visible=true; modeOnly=false;
+#if APIVERSNUM >= 10500
+      SetNeedsFastResponse(true);
 #else
-      Interface->Open(Setup.OSDwidth,osdPos-3);
-      Interface->Clear();
-      if(osdPos<0) Interface->Fill(0,3,Interface->Width(),-osdPos,clrTransparent);
+      needsFastResponse=true;
 #endif
-      needsFastResponse=visible=true;
-      if(file) {
-#if VDRVERSNUM >= 10307
-        display->SetTitle(file->Path());
-#else
-        const char *fileName=file->Path();
-        int n=strlen(fileName);
-        if(n>Width()) {
-          n=n-Width()+4; if(n<0) n=0;
-          char str[72];
-          snprintf(str,sizeof(str),"... %s",fileName+n);
-          Interface->Write(0,0,str);
-          }
-        else Interface->Write(0,0,fileName);
-#endif
-        }
       lastCurrent=lastTotal=-1;
       flush=true;
       }
 
-    if(Current!=lastCurrent) {
-#if VDRVERSNUM >= 10307
-       display->SetProgress(Current, Total);
-       display->SetCurrent(IndexToHMSF(Current));
-       display->SetTotal(IndexToHMSF(Total));
-       bool Play, Forward;
-       int Speed;
-       if(GetReplayMode(Play,Forward,Speed)) 
-         display->SetMode(Play, Forward, Speed);
-#else
-       cProgressBar ProgressBar(Width() * cOsd::CellWidth(), cOsd::LineHeight(), Current, Total);
-       Interface->SetBitmap(0, cOsd::LineHeight(), ProgressBar);
-       Interface->Write(0,2,IndexToHMSF(Current));
-       Interface->Write(-7,2,IndexToHMSF(Total));
-#endif
-       flush=true;
-       lastCurrent=Current; lastTotal=Total;
-       }
+    if(abs(Current-lastCurrent)>12) {
+      if(Total>0) display->SetProgress(Current, Total);
+      display->SetCurrent(IndexToHMSF(Current));
+      display->SetTotal(IndexToHMSF(Total));
+      bool Play, Forward;
+      int Speed;
+      if(GetReplayMode(Play,Forward,Speed)) 
+        display->SetMode(Play, Forward, Speed);
+      ShowTitle();
+      flush=true;
+      lastCurrent=Current; lastTotal=Total;
+      }
     if(flush) 
-#if VDRVERSNUM >= 10307
       Skins.Flush();
-#else
-      Interface->Flush();
-#endif
     ShowMode();
     }
 }
 
-#if VDRVERSNUM < 10307
-#ifdef HAVE_BEAUTYPATCH
-int forwSym[] = { FSYM_FORW,FSYM_FORW1,FSYM_FORW2,FSYM_FORW3 };
-int backSym[] = { FSYM_BACK,FSYM_BACK1,FSYM_BACK2,FSYM_BACK3 };
-#endif
-#endif
-
 void cMPlayerControl::ShowMode(void)
 {
   if(Setup.ShowReplayMode && !jumpactive) {
@@ -274,50 +253,13 @@ void cMPlayerControl::ShowMode(void)
 
        if(!visible) {
          if(NormalPlay) return;
-#if VDRVERSNUM >= 10307
-         display = Skins.Current()->DisplayReplay(modeOnly);
-#else
-         Interface->Open(0,osdPos-1);
-#endif
+         display = Skins.Current()->DisplayReplay(true);
          visible=modeOnly=true;
          }
 
        if(modeOnly && !timeoutShow && NormalPlay) timeoutShow=time(0)+SELECTHIDE_TIMEOUT;
 
-#if VDRVERSNUM >= 10307
        display->SetMode(Play, Forward, Speed);
-#else
-       char buf[16];
-       eDvbFont OldFont;
-#ifdef HAVE_BEAUTYPATCH
-       if(haveBeauty) {
-         int i=0;
-         if(!(Width()&1)) buf[i++]=' ';
-         buf[i]=FSYM_EMPTY; if(Speed>=0 && !Forward) buf[i]=backSym[Speed];
-         i++;
-         buf[i++]=Play?(Speed==-1?FSYM_PLAY:FSYM_EMPTY):FSYM_PAUSE;
-         buf[i]=FSYM_EMPTY; if(Speed>=0 && Forward) buf[i]=forwSym[Speed];
-         i++;
-         if(!(Width()&1)) buf[i++]=' ';
-         buf[i]=0;
-         OldFont = Interface->SetFont(fontSym);
-         }
-       else {
-#endif //HAVE_BEAUTYPATCH
-         const char *Mode;
-         if (Speed == -1) Mode = Play    ? "  >  " : " ||  ";
-         else if (Play)   Mode = Forward ? " X>> " : " <<X ";
-         else             Mode = Forward ? " X|> " : " <|X ";
-         strn0cpy(buf, Mode, sizeof(buf));
-         char *p = strchr(buf, 'X');
-         if(p) *p = Speed > 0 ? '1' + Speed - 1 : ' ';
-         OldFont = Interface->SetFont(fontFix);
-#ifdef HAVE_BEAUTYPATCH
-         }
-#endif //HAVE_BEAUTYPATCH
-       DisplayAtBottom(buf);
-       Interface->SetFont(OldFont);
-#endif //VDRVERSNUM >= 10307
        }
     }
 }
@@ -325,14 +267,10 @@ void cMPlayerControl::ShowMode(void)
 void cMPlayerControl::JumpDisplay(void)
 {
   char buf[64];
-  const char *j=tr("Jump: "), u=jumpmode?'%':'m';
+  const char *j=trVDR("Jump: "), u=jumpmode?'%':'m';
   if(!jumpval) sprintf(buf,"%s- %c",  j,u);
   else         sprintf(buf,"%s%d- %c",j,jumpval,u);
-#if VDRVERSNUM >= 10307
   display->SetJump(buf);
-#else
-  DisplayAtBottom(buf);
-#endif
 }
 
 void cMPlayerControl::JumpProcess(eKeys Key)
@@ -348,7 +286,7 @@ void cMPlayerControl::JumpProcess(eKeys Key)
       break;
     case kBlue:
       jumpmode=!jumpmode; jumpval=0;
-      DisplayAtBottom(0); JumpDisplay();
+      JumpDisplay();
       break;
     case kPlay:
     case kUp:
@@ -372,11 +310,7 @@ void cMPlayerControl::JumpProcess(eKeys Key)
   if(!jumpactive) {
     if(jumphide) Hide();
     else 
-#if VDRVERSNUM >= 10307
       display->SetJump(0);
-#else
-      DisplayAtBottom(0);
-#endif
     }
 }
 
@@ -409,6 +343,7 @@ eOSState cMPlayerControl::ProcessKey(eKeys Key)
         else ShowProgress();
         }
       }
+    else ShowTitle();
 
     if(jumpactive && Key != kNone) {
       JumpProcess(Key);
@@ -435,18 +370,19 @@ eOSState cMPlayerControl::ProcessKey(eKeys Key)
 
       case kRed:     Jump(); break;
 
-      case kGreen|k_Repeat:                      // temporary use
+      case kGreen|k_Repeat:
       case kGreen:   player->SkipSeconds(-60); break;
       case kYellow|k_Repeat:
       case kYellow:  player->SkipSeconds(60); break;
-  //    case kGreen|k_Repeat:                      // reserved for future use
-  //    case kGreen:   player->SkipPrev(); break;
-  //    case kYellow|k_Repeat:
-  //    case kYellow:  player->SkipNext(); break;
 
+      case kNext:    player->SkipTrack(1,MPlayerSetup.PrevNextKeyMode!=0); break;
+      case kPrev:    player->SkipTrack(-1,MPlayerSetup.PrevNextKeyMode!=0); break;
+
+      case kBack:    Hide();
+                     cRemote::CallPlugin(plugin_name);
+                     return osBack;
       case kStop:
-      case kBlue:
-      case kBack:    Hide(); Stop(); return osEnd;
+      case kBlue:    Hide(); Stop(); return osEnd;
 
       default:
         DoShowMode = false;
@@ -454,20 +390,22 @@ eOSState cMPlayerControl::ProcessKey(eKeys Key)
           case kOk: if(visible && !modeOnly) { Hide(); DoShowMode=true; }
                     else ShowTimed();
                     break;
-          case k3:  player->Osd(); break;
-          case k1:  player->AudioDelay(-1); break;
-          case k2:  player->AudioDelay(1); break;
-          case k9:  if(visible && !modeOnly) {
-                      Hide();
-                      osdPos++; if(osdPos>0) osdPos=0;
-                      ShowTimed();
-                      }
+          case kAudio:
+                    player->KeyCmd("switch_audio");
                     break;
-          case k6:  if(visible && !modeOnly) {
-                      Hide();
-                      osdPos--; if(osdPos<-6) osdPos=-6;
-                      ShowTimed();
-                      }
+          case k0:
+          case k1:
+          case k2:
+          case k3:
+          case k4:
+          case k5:
+          case k6:
+          case k7:
+          case k8:
+          case k9:  {
+                    const char *cmd=MPlayerSetup.KeyCmd[Key-k0];
+                    if(cmd[0]) player->KeyCmd(cmd);
+                    }
                     break;
           default:  break;
           }
@@ -490,7 +428,7 @@ public:
 cMenuMPlayAid::cMenuMPlayAid(void)
 :cOsdMenu(tr("MPlayer Audio ID"),20)
 {
-  Add(new cMenuEditIntItem(tr("Audiostream ID"),&MPlayerAid,0,255));
+  Add(new cMenuEditIntItem(tr("Audiostream ID"),&MPlayerAid,-1,255));
   Display();
 }
 
@@ -512,6 +450,7 @@ class cMenuMPlayBrowse : public cMenuBrowse {
 private:
   bool sourcing, aidedit;
   eOSState Source(bool second);
+  eOSState Summary(void);
 protected:
   virtual void SetButtons(void);
 public:
@@ -519,8 +458,10 @@ public:
   virtual eOSState ProcessKey(eKeys Key);
   };
 
+static const char *excl_sum[] = { ".*","*.summary","*.txt","*.nfo",0 };
+
 cMenuMPlayBrowse::cMenuMPlayBrowse(void)
-:cMenuBrowse(MPlaySources.GetSource(),false,false,tr("MPlayer browser"))
+:cMenuBrowse(MPlaySources.GetSource(),false,false,tr("MPlayer browser"),excl_sum)
 {
   sourcing=aidedit=false;
   SetButtons();
@@ -529,8 +470,8 @@ cMenuMPlayBrowse::cMenuMPlayBrowse(void)
 void cMenuMPlayBrowse::SetButtons(void)
 {
   static char blue[12];
-  snprintf(blue,sizeof(blue),"AID:%d",MPlayerAid);
-  SetHelp(tr("Play"), MPlayerSetup.DisableResume ? 0:tr("Rewind"), tr("Source"), blue);
+  snprintf(blue,sizeof(blue),MPlayerAid>=0 ? "AID:%d" : "AID:def",MPlayerAid);
+  SetHelp(trVDR("Button$Play"), MPlayerSetup.ResumeMode ? trVDR("Button$Rewind"):0, tr("Source"), blue);
   Display();
 }
 
@@ -552,6 +493,35 @@ eOSState cMenuMPlayBrowse::Source(bool second)
   return osContinue;
 }
 
+eOSState cMenuMPlayBrowse::Summary(void)
+{
+  cFileObj *item=CurrentItem();
+  if(item && item->Type()==otFile) {
+    static const char *exts[] = { ".summary",".txt",".nfo",0 };
+    for(int i=0; exts[i]; i++) {
+      char buff[4096];
+      strn0cpy(buff,item->FullPath(),sizeof(buff)-20);
+      char *e=&buff[strlen(buff)];
+      strn0cpy(e,exts[i],20);
+      int fd=open(buff,O_RDONLY);
+      *e=0;
+      if(fd<0 && (e=rindex(buff,'.'))) {
+        strn0cpy(e,exts[i],20);
+        fd=open(buff,O_RDONLY);
+        }
+      if(fd>=0) {
+        int r=read(fd,buff,sizeof(buff)-1);
+        close(fd);
+        if(r>0) {
+          buff[r]=0;
+          return AddSubMenu(new cMenuText(tr("Summary"),buff));
+          }
+        }
+      }
+    }
+  return osContinue;
+}
+
 eOSState cMenuMPlayBrowse::ProcessKey(eKeys Key)
 {
   eOSState state=cOsdMenu::ProcessKey(Key);
@@ -580,6 +550,9 @@ eOSState cMenuMPlayBrowse::ProcessKey(eKeys Key)
         aidedit=true;
         state=AddSubMenu(new cMenuMPlayAid);
         break;
+      case k0:
+        state=Summary();
+        break;
       default:
         break;
       }
@@ -595,27 +568,27 @@ eOSState cMenuMPlayBrowse::ProcessKey(eKeys Key)
 
 // --- cPluginMPlayer ----------------------------------------------------------
 
-static const char *VERSION        = PLUGIN_VERSION;
-static const char *DESCRIPTION    = "Media replay via MPlayer";
+static const char *DESCRIPTION    = trNOOP("Media replay via MPlayer");
 static const char *MAINMENUENTRY  = "MPlayer";
 
 class cPluginMPlayer : public cPlugin {
+private:
+  bool ExternalPlay(const char *path, bool test);
 public:
   cPluginMPlayer(void);
   virtual ~cPluginMPlayer();
-  virtual const char *Version(void) { return VERSION; }
+  virtual const char *Version(void) { return PluginVersion; }
   virtual const char *Description(void) { return tr(DESCRIPTION); }
   virtual const char *CommandLineHelp(void);
   virtual bool ProcessArgs(int argc, char *argv[]);
-#if VDRVERSNUM >= 10131
   virtual bool Initialize(void);
-#else
-  virtual bool Start(void);
-#endif
   virtual const char *MainMenuEntry(void);
   virtual cOsdMenu *MainMenuAction(void);
   virtual cMenuSetupPage *SetupMenu(void);
   virtual bool SetupParse(const char *Name, const char *Value);
+  virtual bool Service(const char *Id, void *Data);
+  virtual const char **SVDRPHelpPages(void);
+  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
   };
 
 cPluginMPlayer::cPluginMPlayer(void)
@@ -636,12 +609,18 @@ const char *cPluginMPlayer::CommandLineHelp(void)
   static char *help_str=0;
   
   free(help_str);    //                                     for easier orientation, this is column 80|
-  asprintf(&help_str,"  -m CMD,   --mount=CMD    use CMD to mount/unmount/eject mp3 sources\n"
+  help_str=aprintf(  "  -m CMD,   --mount=CMD    use CMD to mount/unmount/eject mp3 sources\n"
                      "                           (default: %s)\n"
                      "  -M CMD,   --mplayer=CMD  use CMD when calling MPlayer\n"
+                     "                           (default: %s)\n"
+                     "  -S SUB,   --sources=SUB  search sources config in SUB subdirectory\n"
+                     "                           (default: %s)\n"
+                     "  -R DIR,   --resume=DIR   store global resume file in DIR\n"
                      "                           (default: %s)\n",
                      mountscript,
-                     MPlayerCmd
+                     MPlayerCmd,
+                     sourcesSub ? sourcesSub:"none",
+                     globalResumeDir ? globalResumeDir:"video dir"
                      );
   return help_str;
 }
@@ -651,35 +630,42 @@ bool cPluginMPlayer::ProcessArgs(int argc, char *argv[])
   static struct option long_options[] = {
       { "mount",    required_argument, NULL, 'm' },
       { "mplayer",  required_argument, NULL, 'M' },
+      { "sources",  required_argument, NULL, 'S' },
+      { "resume",   required_argument, NULL, 'R' },
       { NULL }
     };
 
   int c, option_index = 0;
-  while((c=getopt_long(argc,argv,"m:M:",long_options,&option_index))!=-1) {
+  while((c=getopt_long(argc,argv,"m:M:S:R:",long_options,&option_index))!=-1) {
     switch (c) {
       case 'm': mountscript=optarg; break;
       case 'M': MPlayerCmd=optarg; break;
+      case 'S': sourcesSub=optarg; break;
+      case 'R': globalResumeDir=optarg; break;
       default:  return false;
       }
     }
   return true;
 }
 
-#if VDRVERSNUM >= 10131
 bool cPluginMPlayer::Initialize(void)
-#else
-bool cPluginMPlayer::Start(void)
-#endif
 {
-  if(!CheckVDRVersion(1,1,16,"mplayer")) return false;
-  i18n_name=Name();
-  MPlaySources.Load(AddDirectory(ConfigDirectory(),"mplayersources.conf"));
+  if(!CheckVDRVersion(1,4,5,"mplayer")) return false;
+  plugin_name="mplayer";
+#if APIVERSNUM < 10507
+  i18n_name="mplayer";
+#else
+  i18n_name="vdr-mplayer";
+#endif
+  MPlaySources.Load(AddDirectory(ConfigDirectory(sourcesSub),"mplayersources.conf"));
   if(MPlaySources.Count()<1) {
     esyslog("ERROR: you must have defined at least one source in mplayersources.conf");
     fprintf(stderr,"No source(s) defined in mplayersources.conf\n");
     return false;
     }
+#if APIVERSNUM < 10507
   RegisterI18n(Phrases);
+#endif
   if(!(status=new cMPlayerStatus)) return false;
   return true;
 }
@@ -703,8 +689,94 @@ bool cPluginMPlayer::SetupParse(const char *Name, const char *Value)
 {
   if(      !strcasecmp(Name, "ControlMode"))  MPlayerSetup.SlaveMode    = atoi(Value);
   else if (!strcasecmp(Name, "HideMainMenu")) MPlayerSetup.HideMainMenu = atoi(Value);
+  else if (!strcasecmp(Name, "ResumeMode"))   MPlayerSetup.ResumeMode   = atoi(Value);
+  else if (!strcasecmp(Name, "PrevNextMode")) MPlayerSetup.PrevNextKeyMode = atoi(Value);
+  else if (!strncasecmp(Name,"KeyCmd", 6) && strlen(Name)==7 && isdigit(Name[6]))
+    strn0cpy(MPlayerSetup.KeyCmd[Name[6]-'0'],Value,sizeof(MPlayerSetup.KeyCmd[0]));
   else return false;
   return true;
 }
 
+bool cPluginMPlayer::ExternalPlay(const char *path, bool test)
+{
+  char real[PATH_MAX+1];
+  if(realpath(path,real)) {
+    cFileSource *src=MPlaySources.FindSource(real);
+    if(src) {
+      cFileObj *item=new cFileObj(src,0,0,otFile);
+      if(item) {
+        item->SplitAndSet(real);
+        if(item->GuessType()) {
+          if(item->Exists()) {
+            if(!test) {
+              cMPlayerControl::SetFile(item,true);
+              cControl::Launch(new cMPlayerControl);
+              cControl::Attach();
+              }
+            delete item;
+            return true;
+            }
+          else dsyslog("MPlayer service: cannot play '%s'",path);
+          }
+        else dsyslog("MPlayer service: GuessType() failed for '%s'",path);
+        delete item;
+        }
+      }
+    else dsyslog("MPlayer service: cannot find source for '%s', real '%s'",path,real);
+    }
+  else if(errno!=ENOENT && errno!=ENOTDIR)
+    esyslog("ERROR: realpath: %s: %s",path,strerror(errno));
+  return false;
+}
+
+bool cPluginMPlayer::Service(const char *Id, void *Data)
+{
+  if(!strcasecmp(Id,"MPlayer-Play-v1")) {
+    if(Data) {
+      struct MPlayerServiceData *msd=(struct MPlayerServiceData *)Data;
+      msd->result=ExternalPlay(msd->data.filename,false);
+      }
+    return true;
+    }
+  else if(!strcasecmp(Id,"MPlayer-Test-v1")) {
+    if(Data) {
+      struct MPlayerServiceData *msd=(struct MPlayerServiceData *)Data;
+      msd->result=ExternalPlay(msd->data.filename,true);
+      }
+    return true;
+    }
+  return false;
+}
+
+const char **cPluginMPlayer::SVDRPHelpPages(void)
+{
+  static const char *HelpPages[] = {
+    "PLAY <filename>\n"
+    "    Triggers playback of file 'filename'.",
+    "TEST <filename>\n"
+    "    Tests is playback of file 'filename' is possible.",
+    NULL
+    };
+  return HelpPages;
+}
+
+cString cPluginMPlayer::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
+{
+  if(!strcasecmp(Command,"PLAY")) {
+    if(*Option) {
+      if(ExternalPlay(Option,false)) return "Playback triggered";
+      else { ReplyCode=550; return "Playback failed"; }
+      }
+    else { ReplyCode=501; return "Missing filename"; }
+    }
+  else if(!strcasecmp(Command,"TEST")) {
+    if(*Option) {
+      if(ExternalPlay(Option,true)) return "Playback possible";
+      else { ReplyCode=550; return "Playback not possible"; }
+      }
+    else { ReplyCode=501; return "Missing filename"; }
+    }
+  return NULL;
+}
+
 VDRPLUGINCREATOR(cPluginMPlayer); // Don't touch this!
