@@ -1,7 +1,7 @@
 /*
  * MP3/MPlayer plugin to VDR (C++)
  *
- * (C) 2001-2004 Stefan Huelswitt <s.huelswitt@gmx.de>
+ * (C) 2001-2009 Stefan Huelswitt <s.huelswitt@gmx.de>
  *
  * This code is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -42,18 +42,20 @@
 #include "decoder.h"
 #include "decoder-core.h"
 
+#ifndef NO_DEBUG
 //#define DEBUG_MODE      // debug playmode changes
 #define DEBUG_BGR       // debug backround scan thread
 #define DEBUG_DELAY 300 // debug write/decode delays
 //#define ACC_DUMP        // dump limiter lookup table to /tmp/limiter
+#endif
 
-#ifdef DEBUG_MODE
+#if !defined(NO_DEBUG) && defined(DEBUG_MODE)
 #define dm(x) { (x); }
 #else
 #define dm(x) ; 
 #endif
 
-#ifdef DEBUG_BGR
+#if !defined(NO_DEBUG) && defined(DEBUG_BGR)
 #define db(x) { (x); }
 #else
 #define db(x) ; 
@@ -346,7 +348,7 @@ cNormalize::cNormalize(void)
 cNormalize::~cNormalize()
 {
 #ifdef USE_FAST_LIMITER
-  delete table;
+  delete[] table;
 #endif
 }
 
@@ -659,6 +661,7 @@ unsigned int cScale::ScaleBlock(unsigned char *data, unsigned int size, unsigned
 class cShuffle {
 private:
   int *shuffle, max;
+  unsigned int seed;
   //
   int Index(int pos);
 public:
@@ -676,7 +679,7 @@ public:
 cShuffle::cShuffle(void)
 {
   shuffle=0; max=0;
-  srand(time(0));
+  seed=time(0);
 }
 
 cShuffle::~cShuffle(void)
@@ -717,7 +720,7 @@ void cShuffle::Shuffle(int num, int curr)
   int in=Index(curr)+1; if(in<0) in=0;
   if((max-in)>=2) {
     for(int i=in ; i<max ; i++) {
-      int ran=(rand() % ((max-in)*4-4))/4; ran+=((ran+in) >= i);
+      int ran=(rand_r(&seed) % ((max-in)*4-4))/4; ran+=((ran+in) >= i);
       int t=shuffle[i];
       shuffle[i]=shuffle[ran+in];
       shuffle[ran+in]=t;
@@ -800,7 +803,9 @@ cPlayManager::~cPlayManager()
 {
   Flush();
   Release();
-  stopscan=true; listCond.Broadcast();
+  listMutex.Lock();
+  stopscan=true; bgCond.Broadcast();
+  listMutex.Unlock();
   Cancel(2);
   delete shuffle;
 }
@@ -809,7 +814,7 @@ void cPlayManager::ThrottleWait(void)
 {
   while(!stopscan && !release && throttle) {
     db(printf("mgr: background scan throttled\n"))
-    listCond.Wait(listMutex);
+    bgCond.Wait(listMutex);
     db(printf("mgr: background scan throttle wakeup\n"))
     }
 }
@@ -817,7 +822,7 @@ void cPlayManager::ThrottleWait(void)
 void cPlayManager::Action(void)
 {
   db(printf("mgr: background scan thread started (pid=%d)\n", getpid()))
-  nice(5);
+  if(nice(5)<0);
   listMutex.Lock();
   while(!stopscan) {
     for(scan=list.First(); !stopscan && !release && scan; scan=list.Next(scan)) {
@@ -868,7 +873,7 @@ void cPlayManager::Action(void)
                       if(si && fi) {
                         si->Level=l;
                         si->Peak=level.GetPeak();
-                        InfoCache.Cache(si,fi,true);
+                        InfoCache.Cache(si,fi);
                         }
                       }
                     }
@@ -890,9 +895,9 @@ void cPlayManager::Action(void)
       pass2=false;
       }
     do {
-      scan=0; release=false; listCond.Broadcast();
+      scan=0; release=false; fgCond.Broadcast();
       db(printf("mgr: background scan idle\n"))
-      listCond.Wait(listMutex);
+      bgCond.Wait(listMutex);
       db(printf("mgr: background scan idle wakeup\n"))
       } while(!stopscan && (release || throttle));
     }
@@ -904,12 +909,13 @@ void cPlayManager::Throttle(bool thr)
 {
   if(MP3Setup.BgrScan) {
     if(!thr && throttle) {
-      db(printf("mgr: bgr-scan -> run (%d)\n",time_ms()))
-      throttle=false;
-      listCond.Broadcast();
+      db(printf("mgr: bgr-scan -> run (%llu)\n",cTimeMs::Now()))
+      listMutex.Lock();
+      throttle=false; bgCond.Broadcast();
+      listMutex.Unlock();
       }
     if(thr && !throttle) {
-      db(printf("mgr: bgr-scan -> throttle (%d)\n",time_ms()))
+      db(printf("mgr: bgr-scan -> throttle (%llu)\n",cTimeMs::Now()))
       throttle=true;
       }
     }
@@ -946,9 +952,10 @@ bool cPlayManager::Info(int num, cMP3PlayInfo *pi)
   bool res=false;
   if(s) {
     strn0cpy(pi->Title,s->Name(),sizeof(pi->Title));
+    strn0cpy(pi->Filename,s->FullPath(),sizeof(pi->Filename));
     cSongInfo *si=s->Info(false);
     if(si && si->HasInfo()) {
-      static char *modestr[] = { "Mono","Dual","Joint-Stereo","Stereo" };
+      static const char *modestr[] = { "Mono","Dual","Joint-Stereo","Stereo" };
 
       if(si->Title)  strn0cpy(pi->Title,si->Title,sizeof(pi->Title));
       if(si->Artist) strn0cpy(pi->Artist,si->Artist,sizeof(pi->Artist));
@@ -977,7 +984,7 @@ void cPlayManager::Add(cPlayList *pl)
   if(real) {
     if(MP3Setup.BgrScan) { stopscan=false; if(!Active()) Start(); }
     else stopscan=true;
-    listCond.Broadcast();
+    bgCond.Broadcast();
     maxIndex=list.Count()-1;
     if(shuffleMode) shuffle->Shuffle(maxIndex+1,currIndex);
     if(!curr) Next();
@@ -997,6 +1004,7 @@ void cPlayManager::Halt(void)
   cMutexLock lock(&listMutex);
   curr=0; currIndex=-1;
   playNew=true;
+  stopscan=true; bgCond.Broadcast();
   NoScan(0);
   NoPlay(0);
 }
@@ -1005,9 +1013,9 @@ void cPlayManager::NoScan(cSong *nono)
 {
   // call with listMutex locked!!
   while((nono && pass2 && scan==nono) || (!nono && scan)) {
-    release=true; listCond.Broadcast();
+    release=true; bgCond.Broadcast();
     d(printf("mgr: waiting for bgr release ... (pass2=%d nono=%p scan=%p)\n",pass2,nono,scan))
-    listCond.Wait(listMutex);
+    fgCond.Wait(listMutex);
     }
 }
 
@@ -1016,7 +1024,7 @@ void cPlayManager::NoPlay(cSong *nono)
   // call with listMutex locked!!
   while((nono && play==nono) || (!nono && play)) {
     playNew=true;
-    listCond.Wait(listMutex);
+    fgCond.Wait(listMutex);
     }
 }
 
@@ -1113,7 +1121,7 @@ cSong *cPlayManager::Current(void)
     playNew=false;
     if(play) d(printf("mgr: playing %s\n",play->Name()))
     else d(printf("mgr: nothing to play\n"))
-    listCond.Broadcast();
+    fgCond.Broadcast();
     }
   return play;
 }
@@ -1133,7 +1141,7 @@ void cPlayManager::Release(void)
 {
   cMutexLock lock(&listMutex);
   play=0;
-  listCond.Broadcast();
+  fgCond.Broadcast();
 }
 
 // --- cOutput -----------------------------------------------------------------
@@ -1155,6 +1163,8 @@ public:
   virtual cFrame *MakeFrame(unsigned int & Samples, const mad_fixed_t **Data, int index, int sr)=0;
   virtual int Output(const unsigned char *Data, int Len, bool SOF)=0;
   virtual bool Poll(void)=0;
+  virtual void Play(void)=0;
+  virtual void Pause(void)=0;
 #ifdef DEBUG
   virtual void Stats(void);
 #endif
@@ -1208,6 +1218,8 @@ public:
   virtual cFrame *MakeFrame(unsigned int & Samples, const mad_fixed_t **Data, int index, int sr);
   virtual int Output(const unsigned char *Data, int Len, bool SOF);
   virtual bool Poll(void);
+  virtual void Play(void);
+  virtual void Pause(void);
   };
 
 cOutputDvb::cOutputDvb(cMP3Player *Player)
@@ -1215,6 +1227,7 @@ cOutputDvb::cOutputDvb(cMP3Player *Player)
 {
   only48khz=MP3Setup.Only48kHz;
   outSr=0;
+  cDevice::PrimaryDevice()->SetCurrentAudioTrack(ttAudio);
   d(printf("mp3-dvb: using DVB output\n"))
 }
 
@@ -1259,7 +1272,7 @@ cFrame *cOutputDvb::MakeFrame(unsigned int & Samples, const mad_fixed_t **Data,
     0xA0, // aLPCM header
     0xFF,
     0x00,
-    0x00,
+    0x04,
     0x00,
     0x01, // 2-channel stereo (n-1)
     0x80  // neutral dynamic range
@@ -1281,6 +1294,9 @@ cFrame *cOutputDvb::MakeFrame(unsigned int & Samples, const mad_fixed_t **Data,
     buff[14+FHS]|=srMode;
     unsigned int outlen=scale.ScaleBlock(buff+sizeof(header)+FHS,FRAMESIZE-sizeof(header)-FHS,Samples,Data[0],Data[1],MP3Setup.AudioMode?amDitherBE:amRoundBE);
     if(outlen) { 
+      // lPCM has 600 fps which is 80 samples at 48kHz per channel
+      // Frame size = (sample_rate * quantization * channels)/4800
+      buff[10+FHS]=outlen*(4800/16/2)/sr;
       outlen+=(sizeof(header)-6);
       buff[4+FHS]=outlen>>8;
       buff[5+FHS]=outlen;
@@ -1291,12 +1307,15 @@ cFrame *cOutputDvb::MakeFrame(unsigned int & Samples, const mad_fixed_t **Data,
   return f;
 }
 
+#ifdef BROKEN_PCM
 #include "player-mp3-sample.c"
+#endif
 
 int cOutputDvb::Output(const unsigned char *Data, int Len, bool SOF)
 {
   int n=0;
   if(SOF) {
+#ifdef BROKEN_PCM
     struct FrameHeader *fh=(struct FrameHeader *)Data;
     if(fh->samplerate!=outSr) {
       if(outSr) {
@@ -1307,7 +1326,7 @@ int cOutputDvb::Output(const unsigned char *Data, int Len, bool SOF)
         int pc=sizeof(testAudio);
         int r;
         do {
-          r=player->PlayVideo(p,pc);
+          r=player->PlayPes(p,pc);
           if(r>0) { p+=r; pc-=r; }
           if(r==0) Poll();
           } while(r>=0 && pc>0);
@@ -1315,10 +1334,11 @@ int cOutputDvb::Output(const unsigned char *Data, int Len, bool SOF)
       outSr=fh->samplerate;
       d(printf("mp3-dvb: output samplerate now %d\n",outSr))
       }
+#endif
     n=FHS;
     Data+=n; Len-=n;
     }
-  int r=player->PlayVideo(Data,Len);
+  int r=player->PlayPes(Data,Len);
   return (r>=0 ? r+n : -1);
 }
 
@@ -1327,6 +1347,20 @@ bool cOutputDvb::Poll(void)
   return player->DevicePoll(poll,500);
 }
 
+void cOutputDvb::Play(void)
+{
+#ifndef BROKEN_PCM
+  player->DevicePlay();
+#endif
+}
+
+void cOutputDvb::Pause(void)
+{
+#ifndef BROKEN_PCM
+  player->DeviceFreeze();
+#endif
+}
+
 // --- cOutputOss --------------------------------------------------------------
 
 #ifdef WITH_OSS
@@ -1349,6 +1383,8 @@ public:
   virtual cFrame *MakeFrame(unsigned int & Samples, const mad_fixed_t **Data, int index, int sr);
   virtual int Output(const unsigned char *Data, int Len, bool SOF);
   virtual bool Poll(void);
+  virtual void Play(void);
+  virtual void Pause(void);
   };
 
 cOutputOss::cOutputOss(cMP3Player *Player)
@@ -1360,14 +1396,21 @@ cOutputOss::cOutputOss(cMP3Player *Player)
 
 cOutputOss::~cOutputOss()
 {
-  close(fd);
+  if(fd>=0) close(fd);
 }
 
 void cOutputOss::Init(void)
 {
   if(fd<0) {
-    fd=open(dspdevice,O_WRONLY);
-    if(fd>=0) poll.Add(fd,true);
+    fd=open(dspdevice,O_WRONLY|O_NONBLOCK);
+    if(fd>=0) {
+      if(fcntl(fd,F_SETFL,0)==0)
+        poll.Add(fd,true);
+      else {
+        esyslog("ERROR: Cannot make dsp device '%s' blocking: %s!",dspdevice,strerror(errno));
+        close(fd); fd=-1;
+        }
+      }
     else esyslog("ERROR: Cannot open dsp device '%s': %s!",dspdevice,strerror(errno));
     }
   cOutput::Init();
@@ -1386,7 +1429,7 @@ bool cOutputOss::Reset(unsigned int sr)
         int real=sr;
         CHECK(ioctl(fd,SNDCTL_DSP_SPEED,&real));
         d(printf("oss: DSP samplerate now %d\n",real))
-        if(abs(real-sr)<sr/50) {
+        if((unsigned int)abs(real-sr)<sr/50) {
           outSr=sr;
           d(printf("mp3-oss: DSP reset done\n"))
           return true;
@@ -1428,15 +1471,22 @@ cFrame *cOutputOss::MakeFrame(unsigned int & Samples, const mad_fixed_t **Data,
 
 int cOutputOss::Output(const unsigned char *Data, int Len, bool SOF)
 {
-  int n=0;
-  if(SOF) {
-    struct FrameHeader *fh=(struct FrameHeader *)Data;
-    if(fh->samplerate!=outSr) Reset(fh->samplerate);
-    n=FHS;
-    Data+=n; Len-=n;
+  if(fd>=0) {
+    int n=0;
+    if(SOF) {
+      struct FrameHeader *fh=(struct FrameHeader *)Data;
+      if(fh->samplerate!=outSr) Reset(fh->samplerate);
+      n=FHS;
+      Data+=n; Len-=n;
+      }
+    if(poll.Poll(0)) {
+      int r=write(fd,Data,Len);
+      if(r<0 && FATALERRNO) return -1;
+      if(r>0) n+=r;
+      }
+    return n;
     }
-  int r=fd>=0 ? write(fd,Data,Len) : -1;
-  return (r>=0 ? r+n : -1);
+  return -1;
 }
 
 bool cOutputOss::Poll(void)
@@ -1444,16 +1494,25 @@ bool cOutputOss::Poll(void)
   return fd>=0 ? poll.Poll(500) : false;
 }
 
+void cOutputOss::Play(void)
+{
+}
+
+void cOutputOss::Pause(void)
+{
+  CHECK(ioctl(fd,SNDCTL_DSP_POST,0));
+}
+
 #endif
 
 // --- cMP3Player --------------------------------------------------------------
 
 cMP3Player::cMP3Player()
-:cPlayer(MP3Setup.BackgrMode ? (MP3Setup.AudioOutMode==AUDIOOUTMODE_DVB ? pmAudioOnly:pmNone) : pmAudioOnlyBlack)
+:cPlayer(MP3Setup.BackgrMode==1 ? pmAudioOnly : pmAudioOnlyBlack)
 {
   active=true; started=false; isStream=false;
   ringBuffer=new cRingBufferFrame(MP3BUFSIZE);
-  rframe=0; pframe=0; decoder=0;
+  rframe=0; pframe=0; decoder=0; output=0;
   playMode=pmStartup; state=msStop;
   playindex=total=0;
 }
@@ -1523,23 +1582,25 @@ void cMP3Player::Action(void)
   bool haslevel=false;
   const unsigned char *p=0;
   int pc=0, readindex=0;
+  bool imageValid=true;
+  cTimeMs imageCheck;
 #ifdef DEBUG
   int beat=0;
 #endif
 #ifdef DEBUG_DELAY
-  int lastwrite=0;
+  cTimeMs lastwrite(2000000);
 #endif
 
   dsyslog("mp3: player thread started (pid=%d)", getpid());
   state=msStop;
   SetPlayMode(pmStopped);
 
-  cOutput *out=0;
+  delete output; output=0;
 #ifdef WITH_OSS
-  if(MP3Setup.AudioOutMode==AUDIOOUTMODE_OSS) out=new cOutputOss(this);
+  if(MP3Setup.AudioOutMode==AUDIOOUTMODE_OSS) output=new cOutputOss(this);
 #endif
-  if(MP3Setup.AudioOutMode==AUDIOOUTMODE_DVB) out=new cOutputDvb(this);
-  if(!out) {
+  if(MP3Setup.AudioOutMode==AUDIOOUTMODE_DVB) output=new cOutputDvb(this);
+  if(!output) {
     d(printf("mp3: audiooutmode mismatch or no output driver\n"))
     esyslog("ERROR: no audio output driver. balling out");
     goto abort;
@@ -1552,7 +1613,7 @@ void cMP3Player::Action(void)
     if(now>=beat) {
       int avail=ringBuffer->Available();
       printf("mp3: heartbeat buffer=%d now=%d\n",avail,now&4095);
-      //out->Stats(); if(haslevel) norm.Stats();
+      //output->Stats(); if(haslevel) norm.Stats();
       beat=now+(avail>(MP3BUFSIZE*10/100) ? (avail<(MP3BUFSIZE*50/100) ? 2 : 20) : 1);
       }
     }
@@ -1561,6 +1622,18 @@ void cMP3Player::Action(void)
     Lock();
 
 next:
+    if(!pframe && playing && !imageValid && imageCheck.Elapsed()) {
+      unsigned char *mem;
+      int len;
+      imageCheck.Set(250);
+      imageValid=playing->Image(mem,len);
+      if(mem) {
+        if(playindex) cCondWait::SleepMs(80); // stillpicture ioctl freezes without this
+        DeviceStillPicture(mem,len);
+        free(mem);
+        }
+      }
+
     bool SOF=false;
     if(!pframe && playMode==pmPlay) {
       pframe=ringBuffer->Get();
@@ -1575,13 +1648,12 @@ next:
     if(pframe) {
 #ifdef DEBUG_DELAY
       {
-      int now=time_ms();
-      if(lastwrite && lastwrite<now-(DEBUG_DELAY+50))
-        printf("mp3: write delayed %d ms\n",now-lastwrite);
-      lastwrite=now;
+      if(lastwrite.TimedOut())
+        printf("mp3: write delayed %llu ms\n",lastwrite.Elapsed());
+      lastwrite.Set(DEBUG_DELAY+50);
       }
 #endif
-      int w=out->Output(p,pc,SOF);
+      int w=output->Output(p,pc,SOF);
       if(w>0) {
         p+=w; pc-=w;
         if(pc<=0) {
@@ -1625,8 +1697,9 @@ next:
                   total=SecondsToFrames(si->Total);
                 }
               d(printf("mp3: isStream=%d levelgood=%d haslevel=%d\n",isStream,levelgood,haslevel))
-              out->Init();
+              output->Init();
               level.Init();
+              if(MP3Setup.BackgrMode==2) imageValid=false;
               state=msDecode;
               break;
               }
@@ -1640,12 +1713,11 @@ next:
         case msDecode:
           {
 #ifdef DEBUG_DELAY
-          int now=time_ms();
+          cTimeMs check(DEBUG_DELAY);
 #endif
           struct Decode *ds=decoder->Decode();
 #ifdef DEBUG_DELAY
-          now=time_ms()-now;
-          if(now>DEBUG_DELAY) printf("mp3: decode delayed %d ms\n",now);
+          if(check.TimedOut()) printf("mp3: decode delayed %llu ms\n",check.Elapsed());
 #endif
           switch(ds->status) {
             case dsPlay:
@@ -1666,7 +1738,7 @@ next:
                   if(si && fi) {
                     si->Level=l;
                     si->Peak=level.GetPeak();
-                    InfoCache.Cache(si,fi,true);
+                    InfoCache.Cache(si,fi);
                     }
                   }
                 }
@@ -1698,7 +1770,7 @@ next:
           data[0]=pcm->samples[0];
           data[1]=pcm->channels>1 ? pcm->samples[1]:0;
 
-          dvbSampleRate=out->SampleRate(pcm->samplerate);
+          dvbSampleRate=output->SampleRate(pcm->samplerate);
           if(dvbSampleRate!=pcm->samplerate) {
             if(resample[0].SetInputRate(pcm->samplerate,dvbSampleRate)) {
               nsamples[0]=resample[0].ResampleBlock(nsamples[0],data[0]);
@@ -1712,7 +1784,7 @@ next:
           state=msOutput;
           break;
         case msOutput:
-          if(nsamples[0]>0) rframe=out->MakeFrame(nsamples[0],data,readindex,dvbSampleRate);
+          if(nsamples[0]>0) rframe=output->MakeFrame(nsamples[0],data,readindex,dvbSampleRate);
           else state=msDecode;
           break;
         case msError:
@@ -1724,10 +1796,10 @@ next:
         case msStop:
           d(printf("mp3: stopping play\n"))
           if(decoder) { decoder->Stop(); decoder=0; }
-          mgr->Release(); playing=0;
+          mgr->Release(); playing=0; imageValid=true;
           levelgood=false;
 #ifdef DEBUG
-          out->Stats(); if(haslevel) norm.Stats();
+          output->Stats(); if(haslevel) norm.Stats();
 #endif
           if(state==msStop) SetPlayMode(pmStopped);
           if(state==msRestart) state=msStart;
@@ -1758,15 +1830,19 @@ next:
 
     if((rframe || state==msWait) && pframe) {
       mgr->Throttle(false);
-      out->Poll();
+      output->Poll();
       }
     else if(playMode!=pmPlay) {
       mgr->Throttle(false);
-      playModeMutex.Lock();
-      if(playMode!=pmPlay) WaitPlayMode(playMode,true);
-      playModeMutex.Unlock();
+      if(!imageValid)
+        cCondWait::SleepMs(100);
+      else {
+        playModeMutex.Lock();
+        if(playMode!=pmPlay) WaitPlayMode(playMode,true);
+        playModeMutex.Unlock();
+        }
 #ifdef DEBUG_DELAY
-      lastwrite=0;
+      lastwrite.Set(2000000);
 #endif
       }
     else if(state!=msWait && ringBuffer->Available()<(MP3BUFSIZE*50/100)) {
@@ -1777,7 +1853,7 @@ next:
 abort:
   Lock();
   delete rframe;
-  delete out;
+  delete output; output=0;
   if(decoder) { decoder->Stop(); decoder=0; }
   mgr->Release(); playing=0;
   SetPlayMode(pmStopped);
@@ -1812,11 +1888,14 @@ void cMP3Player::StopPlay(void) // StopPlay() must be called in locked state!!!
 
 void cMP3Player::Pause(void)
 {
+  Lock();
   if(playMode==pmPaused) Play();
   else if(playMode==pmPlay && !isStream) {
     d(printf("mp3: pause\n"))
+    if(output) output->Pause();
     SetPlayMode(pmPaused);
     }
+  Unlock();
 }
 
 void cMP3Player::Play(void)
@@ -1825,6 +1904,7 @@ void cMP3Player::Play(void)
   if(playMode!=pmPlay) {
     d(printf("mp3: play\n"))
     if(playMode==pmStopped) state=msStart;
+    if(output) output->Play();
     SetPlayMode(pmPlay);
     }
   Unlock();
